@@ -8,6 +8,7 @@ from typing import Optional, Union
 from . import generic, misc
 from .crypt import EnvelopeKeyDecrypter
 from .generic import pdf_name
+from .metadata.model import DocumentMetadata
 from .reader import PdfFileReader, parse_catalog_version
 from .writer import BasePdfFileWriter
 
@@ -29,16 +30,19 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
 
     :param input_stream:
         Input stream to read current revision from.
+    :param strict:
+        Ingest the source file in strict mode. The default is ``True``.
     :param prev:
         Explicitly pass in a PDF reader. This parameter is internal API.
     """
 
     IO_CHUNK_SIZE = 4096
+    _force_write_when_empty = False
 
-    def __init__(self, input_stream, prev: PdfFileReader = None):
+    def __init__(self, input_stream, prev: PdfFileReader = None, strict=True):
         self.input_stream = input_stream
         if prev is None:
-            prev = PdfFileReader(input_stream)
+            prev = PdfFileReader(input_stream, strict=strict)
         self.prev = prev
         self.trailer = trailer = prev.trailer
         root_ref = trailer.raw_get('/Root')
@@ -55,9 +59,14 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
         if self._info is not None:
             self.trailer['/Info'] = self._info
         self._resolves_objs_from = (self, prev)
+        input_ver = self.prev.input_version
+        if input_ver > self.output_version:
+            self.output_version = input_ver
         self.ensure_output_version(self.__class__.output_version)
 
         self.security_handler = prev.security_handler
+        if self.security_handler is not None:
+            self._encrypt = prev.trailer.raw_get("/Encrypt")
 
     @classmethod
     def from_reader(cls, reader: PdfFileReader) -> 'IncrementalPdfFileWriter':
@@ -68,7 +77,7 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
             A :class:`.PdfFileReader` object with a PDF to extend.
         """
 
-        return IncrementalPdfFileWriter(reader.stream, prev=reader)
+        return cls(reader.stream, prev=reader)
 
     def ensure_output_version(self, version):
         # check header
@@ -117,11 +126,15 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
             id1 = generic.ByteStringObject(os.urandom(16))
         return generic.ArrayObject([id1, id2])
 
-    def get_object(self, ido):
+    def get_object(self, ido, as_metadata_stream: bool = False):
         try:
-            return super().get_object(ido)
+            return super().get_object(
+                ido, as_metadata_stream=as_metadata_stream
+            )
         except KeyError:
-            return self.prev.get_object(ido)
+            return self.prev.get_object(
+                ido, as_metadata_stream=as_metadata_stream
+            )
 
     def mark_update(self, obj_ref: Union[generic.Reference,
                                          generic.IndirectObject]):
@@ -172,12 +185,13 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
         trailer[pdf_name('/Prev')] = generic.NumberObject(
             self.prev.last_startxref
         )
-        if self.prev.encrypted and self._encrypt is None:
+        if self.prev.encrypted and \
+                not self.security_handler.is_authenticated():
             # removing encryption in an incremental update is impossible
-            raise ValueError(
-                'Cannot save this document unencrypted. Please call '
-                'encrypt() with the password of the original file '
-                'before calling write().'
+            raise misc.PdfWriteError(
+                'Cannot update this document without encryption credentials '
+                'from the original. Please call encrypt() with the password '
+                'of the original file before calling write().'
             )
 
     def set_custom_trailer_entry(self, key: generic.NameObject,
@@ -200,19 +214,31 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
 
     def write(self, stream):
 
-        if not self.objects:
+        if not self.objects and not self._force_write_when_empty:
             # just write the original and then bail
             self._write_header(stream)
             return
+        self._prep_dom_for_writing()
         super().write(stream)
 
-    def write_updated_section(self, stream):
+    @property
+    def document_meta_view(self) -> DocumentMetadata:
+        return self._meta.view_over(self.prev.document_meta_view)
+
+    def _write_updated_section(self, stream):
         """
         Only write the updated and new objects to the designated output stream.
 
         The new PDF file can then be put together by concatenating the original
         input with the generated output.
 
+        .. danger::
+            The offsets in this output will typically be wrong unless the missing
+            previous section is somehow taken into account.
+
+        .. danger::
+            Object graph finalisation will not run.
+
         :param stream:
             Output stream to write to.
         """
@@ -226,9 +252,10 @@ class IncrementalPdfFileWriter(BasePdfFileWriter):
         operations.
         """
 
+        self._prep_dom_for_writing()
         stream = self.prev.stream
         stream.seek(0, os.SEEK_END)
-        self.write_updated_section(stream)
+        self._write_updated_section(stream)
 
     def encrypt(self, user_pwd):
         """Method to handle updates to encrypted files.
