@@ -1,7 +1,6 @@
 """
 Utilities to deal with signature form fields and their properties in PDF files.
 """
-
 import logging
 from dataclasses import dataclass
 from enum import Enum, Flag, unique
@@ -18,9 +17,11 @@ from pyhanko.pdf_utils.generic import pdf_name, pdf_string
 from pyhanko.pdf_utils.layout import BoxConstraints
 from pyhanko.pdf_utils.misc import (
     OrderedEnum,
+    PdfError,
     PdfReadError,
     PdfWriteError,
     get_and_apply,
+    rd,
 )
 from pyhanko.pdf_utils.rw_common import PdfHandler
 from pyhanko.pdf_utils.writer import BasePdfFileWriter
@@ -31,9 +32,10 @@ __all__ = [
     'SigSeedValueSpec', 'SigCertConstraintFlags', 'SigSeedSubFilter',
     'SeedValueDictVersion', 'SeedLockDocument', 'SigCertKeyUsage',
     'MDPPerm', 'FieldMDPAction', 'FieldMDPSpec',
-    'SignatureFormField',
+    'SignatureFormField', 'InvisSigSettings', 'VisibleSigSettings',
     'enumerate_sig_fields', 'append_signature_field',
-    'ensure_sig_flags', 'prepare_sig_field'
+    'ensure_sig_flags', 'prepare_sig_field',
+    'apply_sig_field_spec_properties'
 ]
 
 logger = logging.getLogger(__name__)
@@ -452,7 +454,7 @@ class SigCertConstraints:
             pdf_dict = pdf_dict.get_object()
         try:
             if pdf_dict['/Type'] != '/SVCert':  # pragma: nocover
-                raise ValueError('Object /Type entry is not /SVCert')
+                raise PdfReadError('Object /Type entry is not /SVCert')
         except KeyError:  # pragma: nocover
             pass
         flags = SigCertConstraintFlags(pdf_dict.get('/Ff', 0))
@@ -893,7 +895,7 @@ class SigSeedValueSpec:
             pdf_dict = pdf_dict.get_object()
         try:
             if pdf_dict['/Type'] != '/SV':  # pragma: nocover
-                raise ValueError('Object /Type entry is not /SV')
+                raise PdfReadError('Object /Type entry is not /SV')
         except KeyError:  # pragma: nocover
             pass
 
@@ -1096,13 +1098,15 @@ class FieldMDPSpec:
         try:
             action = FieldMDPAction(pdf_dict['/Action'])
         except KeyError:  # pragma: nocover
-            raise ValueError("/Action is required.")
+            raise PdfReadError("/Action is required.")
 
         if action != FieldMDPAction.ALL:
             try:
                 fields = pdf_dict['/Fields']
             except KeyError:  # pragma: nocover
-                raise ValueError("/Fields is required when /Action is not /All")
+                raise PdfReadError(
+                    "/Fields is required when /Action is not /All"
+                )
         else:
             fields = None
         return cls(action=action, fields=fields)
@@ -1129,6 +1133,74 @@ class FieldMDPSpec:
         return not lock_result
 
 
+@dataclass(frozen=True)
+class InvisSigSettings:
+    """
+    Invisible signature widget generation settings.
+
+    These settings exist because there is no real way of including an untagged
+    invisible signature in a document that complies with the requirements
+    of both PDF/A-2 (or -3) and PDF/UA-1.
+
+    Compatibility with PDF/A (the default) requires the print flag to be set.
+    Compatibility with PDF/UA requires the hidden flag to be set (which is
+    banned in PDF/A) or the box to be outside the crop box.
+    """
+
+    set_print_flag: bool = True
+    """
+    Set the print flag. Required in PDF/A.
+    """
+
+    set_hidden_flag: bool = False
+    """
+    Set the hidden flag. Required in PDF/UA.
+    """
+
+    box_out_of_bounds: bool = False
+    """
+    Put the box out of bounds (technically, this just makes the box
+    zero-sized with large negative coordinates).
+
+    This is a hack to get around the fact that PDF/UA requires the hidden
+    flag to be set on all in-bounds untagged annotations, and some validators
+    consider [0, 0, 0, 0] to be an in-bounds rectangle if (0, 0) is a point
+    that falls within the crop box.
+    """
+
+
+@dataclass(frozen=True)
+class VisibleSigSettings:
+    """
+    .. versionadded:: 0.14.0
+
+    Additional flags used when setting up visible signature widgets.
+    """
+
+    rotate_with_page: bool = True
+    """
+    Allow the signature widget to rotate with the page if rotation is applied
+    (e.g. by way of the page's ``/Rotate`` entry). Default is ``True``.
+
+    .. note::
+        If ``False``, this will cause the ``NoRotate`` flag to be set.
+    """
+
+    scale_with_page_zoom: bool = True
+    """
+    Allow the signature widget to scale with the page's zoom level.
+    Default is ``True``.
+
+    .. note::
+        If ``False``, this will cause the ``NoZoom`` flag to be set.
+    """
+
+    print_signature: bool = True
+    """
+    Render the signature when the document is printed. Default ``True``.
+    """
+
+
 # TODO deal with fully qualified field names for the signature field
 
 @dataclass(frozen=True)
@@ -1206,6 +1278,25 @@ class SigFieldSpec:
         Instead, these viewers typically substitute their own native widget.
     """
 
+    invis_sig_settings: InvisSigSettings = InvisSigSettings()
+    """
+    Advanced settings to control invisible signature field generation.
+    """
+
+    readable_field_name: Optional[str] = None
+    """
+    Human-readable field name (``/TU`` entry).
+    
+    .. note::
+        This value is commonly rendered as a tooltip in viewers, but also
+        serves an accessibility purpose.
+    """
+
+    visible_sig_settings: VisibleSigSettings = VisibleSigSettings()
+    """
+    Advanced settings to control the generation of visible signature fields.
+    """
+
     def format_lock_dictionary(self) -> Optional[generic.DictionaryObject]:
         if self.field_mdp_spec is None:
             return
@@ -1292,8 +1383,6 @@ def prepare_sig_field(sig_field_name, root,
     .. danger::
         This function is internal API.
     """
-    if sig_field_name is None:  # pragma: nocover
-        raise ValueError
 
     try:
         form = root['/AcroForm']
@@ -1301,7 +1390,7 @@ def prepare_sig_field(sig_field_name, root,
         try:
             fields = form['/Fields']
         except KeyError:
-            raise ValueError('/AcroForm has no /Fields')
+            raise PdfError('/AcroForm has no /Fields')
 
         candidates = enumerate_sig_fields_in(
             fields, with_name=sig_field_name, refs_seen=set()
@@ -1472,7 +1561,9 @@ def append_signature_field(pdf_out: BasePdfFileWriter,
         sig_field_spec.sig_field_name, root, update_writer=pdf_out,
         existing_fields_only=False,
         box=sig_field_spec.box, include_on_page=page_ref,
-        combine_annotation=sig_field_spec.combine_annotation
+        combine_annotation=sig_field_spec.combine_annotation,
+        invis_settings=sig_field_spec.invis_sig_settings,
+        visible_settings=sig_field_spec.visible_sig_settings
     )
     ensure_sig_flags(writer=pdf_out, lock_sig_flags=False)
     if not field_created:
@@ -1482,12 +1573,9 @@ def append_signature_field(pdf_out: BasePdfFileWriter,
         )
 
     sig_field = sig_field_ref.get_object()
-    if sig_field_spec.seed_value_dict is not None:
-        # /SV must be an indirect reference as per the spec
-        sv_ref = pdf_out.add_object(
-            sig_field_spec.seed_value_dict.as_pdf_object()
-        )
-        sig_field[pdf_name('/SV')] = sv_ref
+    apply_sig_field_spec_properties(
+        pdf_out, sig_field=sig_field, sig_field_spec=sig_field_spec
+    )
 
     if sig_field_spec.box is not None:
         llx, lly, urx, ury = sig_field_spec.box
@@ -1515,6 +1603,24 @@ def append_signature_field(pdf_out: BasePdfFileWriter,
                 ).as_form_xobject()
             ap_dict[pdf_name('/N')] = pdf_out.add_object(ap_stream)
 
+
+def apply_sig_field_spec_properties(
+        pdf_out: BasePdfFileWriter, sig_field: generic.DictionaryObject,
+        sig_field_spec: SigFieldSpec):
+    """
+    Internal function to apply field spec properties to a newly created field.
+    """
+
+    if sig_field_spec.readable_field_name is not None:
+        sig_field[pdf_name('/TU')] \
+            = generic.TextStringObject(sig_field_spec.readable_field_name)
+    if sig_field_spec.seed_value_dict is not None:
+        # /SV must be an indirect reference as per the spec
+        sv_ref = pdf_out.add_object(
+            sig_field_spec.seed_value_dict.as_pdf_object()
+        )
+        sig_field[pdf_name('/SV')] = sv_ref
+
     lock = sig_field_spec.format_lock_dictionary()
     if lock is not None:
         sig_field[pdf_name('/Lock')] = pdf_out.add_object(lock)
@@ -1523,13 +1629,17 @@ def append_signature_field(pdf_out: BasePdfFileWriter,
 class SignatureFormField(generic.DictionaryObject):
     def __init__(self, field_name, *, box=None, include_on_page=None,
                  combine_annotation=True,
-                 # this sets the "print" and "lock" bits
-                 annot_flags=0b10000100):
+                 invis_settings: InvisSigSettings = InvisSigSettings(),
+                 visible_settings: VisibleSigSettings = VisibleSigSettings(),
+                 annot_flags=None):
 
         if box is not None:
-            rect = list(map(generic.FloatObject, box))
+            rect = [generic.FloatObject(rd(x)) for x in box]
+            invisible = not (abs(box[0] - box[2]) and abs(box[1] - box[3]))
         else:
-            rect = [generic.FloatObject(0)] * 4
+            coord = -9999 if invis_settings.box_out_of_bounds else 0
+            rect = [generic.FloatObject(coord)] * 4
+            invisible = True
 
         super().__init__({
             # Signature field properties
@@ -1546,6 +1656,23 @@ class SignatureFormField(generic.DictionaryObject):
         # Annotation properties: bare minimum
         annot_dict['/Type'] = pdf_name('/Annot')
         annot_dict['/Subtype'] = pdf_name('/Widget')
+
+        if annot_flags is None:
+            # this sets the "lock" bit
+            annot_flags = 0b10000000
+            if invisible:
+                if invis_settings.set_hidden_flag:
+                    annot_flags |= 0b10
+                if invis_settings.set_print_flag:
+                    annot_flags |= 0b100
+            else:
+                if visible_settings.print_signature:
+                    annot_flags |= 0b100
+                if not visible_settings.scale_with_page_zoom:
+                    annot_flags |= 0b1000
+                if not visible_settings.rotate_with_page:
+                    annot_flags |= 0b10000
+
         annot_dict['/F'] = generic.NumberObject(annot_flags)
         annot_dict['/Rect'] = generic.ArrayObject(rect)
 
