@@ -1,9 +1,9 @@
-from .api import TimeStamper, TimestampSignatureStatus
+from .api import TimeStamper
 from .common_utils import TimestampRequestError
 from .dummy_client import DummyTimeStamper
 from .requests_client import HTTPTimeStamper
 
 __all__ = [
     'TimeStamper', 'HTTPTimeStamper', 'DummyTimeStamper',
-    'TimestampSignatureStatus', 'TimestampRequestError'
+    'TimestampRequestError'
 ]
