@@ -5,13 +5,11 @@ from datetime import datetime
 from typing import List, Optional, Union
 
 from asn1crypto import cms, x509
-from cryptography.hazmat.primitives import hashes
 from pyhanko_certvalidator import ValidationContext
 from pyhanko_certvalidator.path import ValidationPath
 
 from pyhanko.pdf_utils import generic, misc
 from pyhanko.pdf_utils.generic import pdf_name
-from pyhanko.pdf_utils.misc import DEFAULT_CHUNK_SIZE
 from pyhanko.pdf_utils.reader import PdfFileReader, process_data_at_eof
 from pyhanko.sign.diff_analysis import (
     DEFAULT_DIFF_POLICY,
@@ -30,9 +28,8 @@ from pyhanko.sign.fields import (
 )
 from pyhanko.sign.general import (
     UnacceptableSignerError,
-    extract_certificate_info,
+    byte_range_digest,
     extract_signer_info,
-    get_pyca_cryptography_hash,
 )
 
 from .errors import (
@@ -45,6 +42,7 @@ from .generic_cms import (
     collect_signer_attr_status,
     collect_timing_info,
     compute_signature_tst_digest,
+    extract_certs_for_validation,
     extract_self_reported_ts,
     extract_tst_data,
     validate_tst_signed_data,
@@ -59,7 +57,7 @@ from .status import (
 __all__ = [
     'EmbeddedPdfSignature', 'DocMDPInfo', 'read_certification_data',
     'async_validate_pdf_signature', 'async_validate_pdf_timestamp',
-    'report_seed_value_validation'
+    'report_seed_value_validation', 'extract_contents'
 ]
 
 logger = logging.getLogger(__name__)
@@ -90,6 +88,30 @@ def _extract_docmdp_for_sig(signature_obj) -> Optional[MDPPerm]:
         )
 
 
+def extract_contents(sig_object: generic.DictionaryObject) -> bytes:
+    """
+    Internal function to extract the (DER-encoded) signature bytes from a PDF
+    signature dictionary.
+
+    :param sig_object:
+        A signature dictionary.
+    :return:
+        The extracted contents as a byte string.
+    """
+
+    try:
+        cms_content = sig_object.raw_get(
+            '/Contents', decrypt=generic.EncryptedObjAccess.RAW
+        )
+    except KeyError:
+        raise misc.PdfReadError('Could not read /Contents entry in signature')
+
+    if not isinstance(cms_content,
+                      (generic.TextStringObject, generic.ByteStringObject)):
+        raise misc.PdfReadError('/Contents must be string-like')
+    return cms_content.original_bytes
+
+
 # TODO clarify in docs that "external timestamp" is always None when dealing
 #  with a /DocTimeStamp, since there the timestamp token is simply the entire
 #  signature object
@@ -123,32 +145,19 @@ class EmbeddedPdfSignature:
         self.sig_object = sig_object = sig_object_ref.get_object()
         assert isinstance(sig_object, generic.DictionaryObject)
         try:
-            pkcs7_content = sig_object.raw_get('/Contents', decrypt=False)
-            self.byte_range = sig_object['/ByteRange']
+            self.byte_range = sig_object.raw_get('/ByteRange')
         except KeyError:
             raise misc.PdfReadError(
-                'Signature PDF object is not correctly formatted'
+                'Could not read /ByteRange entry in signature'
             )
+        self.pkcs7_content = cms_content = extract_contents(sig_object)
 
-        # we need the pkcs7_content raw, so we need to deencapsulate a couple
-        # pieces of data here.
-        if isinstance(pkcs7_content, generic.DecryptedObjectProxy):
-            # it was a direct reference, so just grab the raw one
-            pkcs7_content = pkcs7_content.raw_object
-        elif isinstance(pkcs7_content, generic.IndirectObject):
-            raise misc.PdfReadError("/Contents in signature must be direct")
-
-        if not isinstance(pkcs7_content,
-                          (generic.TextStringObject, generic.ByteStringObject)):
-            raise misc.PdfReadError('/Contents must be string-like')
-        self.pkcs7_content = pkcs7_content
-
-        message = cms.ContentInfo.load(pkcs7_content.original_bytes)
+        message = cms.ContentInfo.load(cms_content)
         signed_data = message['content']
         self.signed_data: cms.SignedData = signed_data
 
         self.signer_info = extract_signer_info(signed_data)
-        self._sd_cert_info = extract_certificate_info(signed_data)
+        self._sd_cert_info = None
 
         # The PDF standard does not define a way to specify the digest algorithm
         # used other than this one.
@@ -196,11 +205,16 @@ class EmbeddedPdfSignature:
         self._integrity_checked = False
         self.fq_name = fq_name
 
+    def _init_cert_info(self):
+        if self._sd_cert_info is None:
+            self._sd_cert_info = extract_certs_for_validation(self.signed_data)
+
     @property
     def embedded_attr_certs(self) -> List[cms.AttributeCertificateV2]:
         """
         Embedded attribute certificates.
         """
+        self._init_cert_info()
         return list(self._sd_cert_info.attribute_certs)
 
     @property
@@ -208,6 +222,7 @@ class EmbeddedPdfSignature:
         """
         Embedded X.509 certificates, excluding than that of the signer.
         """
+        self._init_cert_info()
         return list(self._sd_cert_info.other_certs)
 
     @property
@@ -215,6 +230,7 @@ class EmbeddedPdfSignature:
         """
         Certificate of the signer.
         """
+        self._init_cert_info()
         return self._sd_cert_info.signer_cert
 
     @property
@@ -275,6 +291,7 @@ class EmbeddedPdfSignature:
         :param skip_diff:
             If ``True``, skip the difference analysis step entirely.
         """
+        self._enforce_hybrid_xref_policy()
         self.compute_digest()
         self.compute_tst_digest()
 
@@ -405,22 +422,11 @@ class EmbeddedPdfSignature:
         if self.external_digest is not None:
             return self.external_digest
 
-        md_spec = get_pyca_cryptography_hash(self.external_md_algorithm)
-        md = hashes.Hash(md_spec)
-        stream = self.reader.stream
-
-        # compute the digest
-        # here, we allow arbitrary byte ranges
-        # for the coverage check, we'll impose more constraints
-        total_len = 0
-        chunk_buf = bytearray(DEFAULT_CHUNK_SIZE)
-        for lo, chunk_len in misc.pair_iter(self.byte_range):
-            stream.seek(lo)
-            misc.chunked_digest(chunk_buf, stream, md, max_read=chunk_len)
-            total_len += chunk_len
-
-        self.total_len = total_len
-        self.external_digest = digest = md.finalize()
+        self.total_len, digest = byte_range_digest(
+            self.reader.stream, byte_range=self.byte_range,
+            md_algorithm=self.external_md_algorithm,
+        )
+        self.external_digest = digest
         return digest
 
     def compute_tst_digest(self) -> Optional[bytes]:
@@ -512,12 +518,20 @@ class EmbeddedPdfSignature:
         # ... then check (1) for all revisions up to and including
         # signed_revision
         for revision in range(signed_rev + 1):
-            xref_start, xref_end = xref_cache.get_xref_container_info(revision)
-            if xref_end > signed_zone_len:
+            xref_meta = xref_cache.get_xref_container_info(revision)
+            if xref_meta.end_location > signed_zone_len:
                 return SignatureCoverageLevel.CONTIGUOUS_BLOCK_FROM_START
 
         return SignatureCoverageLevel.ENTIRE_REVISION
 
+    def _enforce_hybrid_xref_policy(self):
+        reader = self.reader
+        if reader.strict and reader.xrefs.hybrid_xrefs_present:
+            raise SignatureValidationError(
+                "Settings do not permit validation of signatures in "
+                "hybrid-reference files."
+            )
+
     def evaluate_modifications(self, diff_policy: DiffPolicy) \
             -> Union[DiffResult, SuspiciousModification]:
         """
@@ -878,7 +892,8 @@ async def async_validate_pdf_timestamp(
                            embedded_sig: EmbeddedPdfSignature,
                            validation_context: ValidationContext = None,
                            diff_policy: DiffPolicy = None,
-                           skip_diff: bool = False) -> DocumentTimestampStatus:
+                           skip_diff: bool = False,
+                           ) -> DocumentTimestampStatus:
     """
     .. versionadded:: 0.9.0
 
